@@ -63,8 +63,8 @@
 # define MZ_USE_SINGLE_FLOATS
 #endif
 
-/* gcc defines __SSE_MATH__ when SSE floating point is enabled: */
-#ifdef __SSE_MATH__
+/* gcc defines __SSE2_MATH__ when SSE2 floating point is enabled: */
+#ifdef __SSE2_MATH__
 # define C_COMPILER_USES_SSE 1
 #endif
 
@@ -957,7 +957,12 @@ typedef struct Scheme_Env Scheme_Env;
 /*========================================================================*/
 
 #ifdef USE_MZ_SETJMP
+# if defined(_WIN64)
+#  define USE_MZ_SETJMP_INDIRECT
+typedef intptr_t mz_pre_jmp_buf[31];
+# else
 typedef intptr_t mz_pre_jmp_buf[8];
+# endif
 #else
 # define mz_pre_jmp_buf jmp_buf
 #endif
@@ -1008,6 +1013,23 @@ typedef struct Scheme_Continuation_Jump_State {
   char is_kill, is_escape, skip_dws;
 } Scheme_Continuation_Jump_State;
 
+#ifdef USE_MZ_SETJMP_INDIRECT
+/* Needed to avoid a direct reference to scheme_mz_setjmp,
+   which might be implemented in assembly and incompatible
+   with delayloading: */
+typedef int (*Scheme_Setjmp_Proc)(mz_pre_jmp_buf);
+# ifndef MZ_XFORM
+#  define scheme_call_mz_setjmp(s) ((scheme_get_mz_setjmp())(s))
+# else
+#  define scheme_call_mz_setjmp(s) scheme_mz_setjmp_post_xform(s)
+# endif
+#else
+# ifdef USE_MZ_SETJMP
+#  define scheme_call_mz_setjmp(s) scheme_mz_setjmp(s)
+# endif
+typedef int (*Scheme_Setjmp_Proc)(void*);
+#endif
+
 /* A mark position is in odd number, so that it can be
    viewed as a pointer (i.e., a fixnum): */
 #define MZ_MARK_POS_TYPE intptr_t
@@ -1192,6 +1214,10 @@ typedef struct Scheme_Thread {
     } k;
   } ku;
 
+  /* To pass the current procedure from one chaperone
+     layer to the next: */
+  Scheme_Object *self_for_proc_chaperone;
+
   short suspend_break;
   short external_break;
 
@@ -1378,6 +1404,7 @@ enum {
 
   MZCONFIG_CURRENT_DIRECTORY,
   MZCONFIG_CURRENT_ENV_VARS,
+  MZCONFIG_FORCE_DELETE_PERMS,
 
   MZCONFIG_CURRENT_USER_DIRECTORY,
 
@@ -1686,20 +1713,20 @@ MZ_EXTERN Scheme_Object *scheme_eval_waiting;
 #ifndef USE_MZ_SETJMP
 # ifdef USE_UNDERSCORE_SETJMP
 #  define scheme_mz_longjmp(b, v) _longjmp(b, v)
-#  define scheme_mz_setjmp(b) _setjmp(b)
+#  define scheme_call_mz_setjmp(b) _setjmp(b)
 # else
 #  define scheme_mz_longjmp(b, v) longjmp(b, v)
-#  define scheme_mz_setjmp(b) setjmp(b)
+#  define scheme_call_mz_setjmp(b) setjmp(b)
 # endif
 #endif
 
 #ifdef MZ_USE_JIT
 MZ_EXTERN void scheme_jit_longjmp(mz_jit_jmp_buf b, int v);
 MZ_EXTERN void scheme_jit_setjmp_prepare(mz_jit_jmp_buf b);
-# define scheme_jit_setjmp(b) (scheme_jit_setjmp_prepare(b), scheme_mz_setjmp((b)->jb))
+# define scheme_jit_setjmp(b) (scheme_jit_setjmp_prepare(b), scheme_call_mz_setjmp((b)->jb))
 #else
 # define scheme_jit_longjmp(b, v) scheme_mz_longjmp(b, v) 
-# define scheme_jit_setjmp(b) scheme_mz_setjmp(b) 
+# define scheme_jit_setjmp(b) scheme_call_mz_setjmp(b)
 #endif
 
 #ifdef MZ_PRECISE_GC
@@ -2106,8 +2133,8 @@ extern Scheme_Extension_Table *scheme_extension_table;
 # define MZ_FD_CLR(n, p) scheme_fdclr(p, n)
 # define MZ_FD_ISSET(n, p) scheme_fdisset(p, n)
 #else
-# define MZ_GET_FDSET(p, n) ((void *)(((fd_set *)p) + n))
-# define MZ_FD_ZERO(p) FD_ZERO((fd_set *)(p))
+# define MZ_GET_FDSET(p, n) ((void *)(((fd_set *)p) XFORM_OK_PLUS n))
+# define MZ_FD_ZERO(p) XFORM_HIDE_EXPR(FD_ZERO((fd_set *)(p)))
 # define MZ_FD_SET(n, p) FD_SET(n, (fd_set *)(p))
 # define MZ_FD_CLR(n, p) FD_CLR(n, (fd_set *)(p))
 # define MZ_FD_ISSET(n, p) FD_ISSET(n, (fd_set *)(p))
