@@ -49,20 +49,28 @@ extern "C" {
 #endif
 
 /* Set up MZ_EXTERN for DLL build */
-#if (defined(__WIN32__) || defined(WIN32) || defined(_WIN32)) \
+#if (defined(__WIN32__) || defined(WIN32) || defined(_WIN32) || defined(__CYGWIN32__)) \
     && !defined(LINK_EXTENSIONS_BY_TABLE) \
     && !defined(SCHEME_EMBEDDED_NO_DLL)
 # define MZ_DLLIMPORT __declspec(dllimport)
 # define MZ_DLLEXPORT __declspec(dllexport)
-# if defined(__mzscheme_private__) || defined(__MINGW32_DELAY_LOAD__)
+# if (defined(__mzscheme_private__) || defined(__MINGW32_DELAY_LOAD__) \
+      || defined(MZ_NO_LIBRACKET_DLL)                                  \
+      || (defined(__CYGWIN32__) && !defined(MZ_USES_SHARED_LIB)))
 #  define MZ_DLLSPEC __declspec(dllexport)
 # else
 #  define MZ_DLLSPEC __declspec(dllimport)
 # endif
+# if (defined(__CYGWIN32__) && !defined(MZ_USES_SHARED_LIB)) || defined(MZ_PRECISE_GC)
+#  define MZGC_DLLIMPORT
+# else
+#  define MZGC_DLLIMPORT MZ_DLLIMPORT
+# endif
 #else
 # define MZ_DLLSPEC
 # define MZ_DLLIMPORT
 # define MZ_DLLEXPORT
+# define MZGC_DLLIMPORT
 #endif
 
 #define MZ_EXTERN extern MZ_DLLSPEC
@@ -283,7 +291,6 @@ typedef struct Thread_Local_Variables {
   struct Scheme_Logger *scheme_future_logger_;
   struct Scheme_Logger *scheme_place_logger_;
   int intdef_counter_;
-  int builtin_ref_counter_;
   int env_uid_counter_;
   int scheme_overflow_count_;
   struct Scheme_Object *original_pwd_;
@@ -320,6 +327,7 @@ typedef struct Thread_Local_Variables {
   volatile short delayed_break_ready_;
   struct Scheme_Thread *main_break_target_thread_;
   intptr_t scheme_code_page_total_;
+  intptr_t max_gc_pre_used_bytes_;
   int locale_on_;
   void *current_locale_name_ptr_;
   int gensym_counter_;
@@ -360,6 +368,7 @@ typedef struct Thread_Local_Variables {
   int atomic_timeout_atomic_level_;
   void *scheme_inotify_server_;
   struct Scheme_Object *configuration_callback_cache_[2];
+  struct FFI_Orig_Place_Call *cached_orig_place_todo_;
 } Thread_Local_Variables;
 
 #if defined(IMPLEMENT_THREAD_LOCAL_VIA_PTHREADS)
@@ -668,7 +677,6 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define scheme_future_logger XOA (scheme_get_thread_local_variables()->scheme_future_logger_)
 #define scheme_place_logger XOA (scheme_get_thread_local_variables()->scheme_place_logger_)
 #define intdef_counter XOA (scheme_get_thread_local_variables()->intdef_counter_)
-#define builtin_ref_counter XOA (scheme_get_thread_local_variables()->builtin_ref_counter_)
 #define env_uid_counter XOA (scheme_get_thread_local_variables()->env_uid_counter_)
 #define scheme_overflow_count XOA (scheme_get_thread_local_variables()->scheme_overflow_count_)
 #define original_pwd XOA (scheme_get_thread_local_variables()->original_pwd_)
@@ -705,6 +713,7 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define delayed_break_ready XOA (scheme_get_thread_local_variables()->delayed_break_ready_)
 #define main_break_target_thread XOA (scheme_get_thread_local_variables()->main_break_target_thread_)
 #define scheme_code_page_total XOA (scheme_get_thread_local_variables()->scheme_code_page_total_)
+#define max_gc_pre_used_bytes XOA (scheme_get_thread_local_variables()->max_gc_pre_used_bytes_)
 #define locale_on XOA (scheme_get_thread_local_variables()->locale_on_)
 #define current_locale_name_ptr XOA (scheme_get_thread_local_variables()->current_locale_name_ptr_)
 #define gensym_counter XOA (scheme_get_thread_local_variables()->gensym_counter_)
@@ -745,7 +754,7 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define atomic_timeout_atomic_level XOA (scheme_get_thread_local_variables()->atomic_timeout_atomic_level_)
 #define scheme_inotify_server XOA (scheme_get_thread_local_variables()->scheme_inotify_server_)
 #define configuration_callback_cache XOA (scheme_get_thread_local_variables()->configuration_callback_cache_)
-
+#define cached_orig_place_todo XOA (scheme_get_thread_local_variables()->cached_orig_place_todo_)
 
 /* **************************************** */
 
