@@ -1,4 +1,4 @@
-/* 
+/*
    Configuration for compiling Racket
 
    If you want to set all the flags externally (on the command line
@@ -6,8 +6,8 @@
    FLAGS_ALREADY_SET, and this file will be ignored.
 
    The best flag settings are already provided for some auto-detected
-   architecture/system/compilers. Otherwise, the default settings 
-   are generic Unix.  Send other architecture/system/compiler-specific 
+   architecture/system/compilers. Otherwise, the default settings
+   are generic Unix.  Send other architecture/system/compiler-specific
    info to "racket@racket-lang.org".
 */
 
@@ -114,8 +114,14 @@
 # define USE_ON_EXIT_FOR_ATEXIT
 # endif
 
+# define PREFER_MMAP_LARGE_BLOCKS
+
 # define FMOD_CAN_RETURN_POS_ZERO
 
+# ifdef _POSIX_PTHREAD_SEMANTICS
+#  define SUBPROCESS_USE_FORK1
+# endif
+
 # ifdef i386
 #  define MZ_USE_JIT_I386
 #  define MZ_JIT_USE_MPROTECT
@@ -156,39 +162,49 @@
 
 #if defined(linux)
 
+# ifdef __ANDROID__
+#  define SPLS_LINUX "android"
+# else
+#  define SPLS_LINUX "linux"
+# endif
+
 # if defined(i386)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "i386-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "i386-"SPLS_LINUX
 #  define REGISTER_POOR_MACHINE
 #  define MZ_TRY_EXTFLONUMS
 #  define ASM_DBLPREC_CONTROL_87
 # endif
 # if defined(powerpc)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-"SPLS_LINUX
 # endif
 # if defined(__mc68000__)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "m68k-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "m68k-"SPLS_LINUX
 # endif
 # if defined(mips)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "mips-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "mips-"SPLS_LINUX
 # endif
 # if defined(__alpha__)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "alpha-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "alpha-"SPLS_LINUX
 # endif
 # if defined(__hppa__)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "hppa-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "hppa-"SPLS_LINUX
 # endif
 # if defined(__sparc__)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "sparc-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "sparc-"SPLS_LINUX
 #  define FLUSH_SPARC_REGISTER_WINDOWS
 # endif
+# if defined(__arm__) || defined(__thumb__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "arm-"SPLS_LINUX
+#  define FFI_CALLBACK_NEED_INT_CLEAR
+# endif
 # if defined(__x86_64__)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "x86_64-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "x86_64-"SPLS_LINUX
 #  define REGISTER_POOR_MACHINE
 #  define ASM_DBLPREC_CONTROL_87
 #  define MZ_TRY_EXTFLONUMS
 # endif
 # ifndef SCHEME_PLATFORM_LIBRARY_SUBPATH
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "unknown-linux"
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "unknown-"SPLS_LINUX
 # endif
 
 # include "uconfig.h"
@@ -215,11 +231,17 @@
 
 # define MZ_TCP_LISTEN_IPV6_ONLY_SOCKOPT
 
+# ifdef __ANDROID__
+#  define USE_FCNTL_O_NONBLOCK
+#  define PROTOENT_IS_INT IPPROTO_TCP
+# endif
+
 # define FLAGS_ALREADY_SET
 
 #if defined(i386)
 # define MZ_USE_JIT_I386
 # define MZ_JIT_USE_MPROTECT
+# define MZ_USE_DWARF_LIBUNWIND
 #endif
 #if defined(__x86_64__)
 # define MZ_USE_JIT_X86_64
@@ -229,6 +251,12 @@
 #if defined(powerpc)
 # define MZ_USE_JIT_PPC
 #endif
+# if defined(__arm__)
+# define MZ_USE_JIT_ARM
+# ifndef __ANDROID__
+#  define MZ_USE_DWARF_LIBUNWIND
+# endif
+#endif
 
 #endif
 
@@ -284,35 +312,45 @@
 
 #endif
 
-  /************** x86/OpenBSD with gcc ****************/
+  /************** OpenBSD with gcc ****************/
               /* Thanks to Bengt Kleberg */
 
-#if defined(__OpenBSD__) && (defined(__i386__) || defined(i386) || defined(__x86_64__))
+#if defined(__OpenBSD__) && !defined(__Bitrig__)
 
 # if defined(__x86_64__)
 #  define SCHEME_PLATFORM_LIBRARY_SUBPATH "x86_64-openbsd"
-# else
+# elif defined(__i386__) || defined(i386)
 #  define SCHEME_PLATFORM_LIBRARY_SUBPATH "i386-openbsd"
-# endif
-
-# include <sys/param.h>
-# if OpenBSD < 201211
-/* This is needed for (pre-5.2) userspace threads: */
-#  define ASSUME_FIXED_STACK_SIZE
-#  define FIXED_STACK_SIZE 1048576
+# elif defined(__mips64__)
+#  if defined(__MIPSEL__)
+#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "mips64el-openbsd"
+#  else
+#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "mips64-openbsd"
+#  endif
+# elif defined(__powerpc__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-openbsd"
+# elif defined(__sparc64__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "sparc64-openbsd"
+/* ARMv7 is a WIP platform on OpenBSD, probably broken here and there */
+# elif defined(__arm__) || defined(__thumb__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "arm-openbsd"
+# elif defined(__hppa__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "hppa-openbsd"
+# else
+#  error Unported platform.
 # endif
 
 # include "uconfig.h"
 # undef HAS_STANDARD_IOB
 # define HAS_BSD_IOB
+# undef UNIX_FIND_STACK_BOUNDS
+# define PTHREAD_STACKSEG_FIND_STACK_BOUNDS
 
 /* Default UNIX_STACK_MAXIMUM is too big for a non-root user. */
 # undef UNIX_STACK_MAXIMUM
 # define UNIX_STACK_MAXIMUM 4194304
 
-#ifndef __ELF__
 # define UNDERSCORE_DYNLOAD_SYMBOL_PREFIX
-#endif
 
 # define USE_DLOPEN_GLOBAL_BY_DEFAULT
 
@@ -322,31 +360,91 @@
 
 # define SIGSET_IS_SIGNAL
 
-# define REGISTER_POOR_MACHINE
-
 # define USE_TM_GMTOFF_FIELD
 # define USE_TM_ZONE_FIELD
 
-#if defined(__x86_64__)
-# define MZ_USE_JIT_X86_64
-# define MZ_JIT_USE_MPROTECT
-# define MZ_TRY_EXTFLONUMS
-#elif defined(__i386__)
-# define MZ_USE_JIT_I386
-# define MZ_JIT_USE_MPROTECT
-# define MZ_TRY_EXTFLONUMS
-#else
-# error Unported platform.
-#endif
+# define USE_MAP_ANON
+
+# if defined(__x86_64__)
+#  define MZ_USE_JIT_X86_64
+#  define MZ_JIT_USE_MPROTECT
+#  define MZ_TRY_EXTFLONUMS
+# elif defined(__i386__)
+#  define MZ_USE_JIT_I386
+#  define MZ_JIT_USE_MPROTECT
+#  define MZ_TRY_EXTFLONUMS
+#  define REGISTER_POOR_MACHINE
+# elif defined(__powerpc__)
+#  define MZ_USE_JIT_PPC
+# elif defined(__sparc64__)
+#  define FLUSH_SPARC_REGISTER_WINDOWS
+# elif defined(__arm__)
+#  define MZ_USE_JIT_ARM
+#  define FFI_CALLBACK_NEED_INT_CLEAR
+# endif
 
 # define FLAGS_ALREADY_SET
 
 #endif
 
-  /************** x86/FreeBSD with gcc ****************/
+  /************** Bitrig with clang ****************/
+/*
+   Bitrig and OpenBSD are pretty similar in some aspects. If you are modifying
+   any of the next values, please test also the same changes on OpenBSD.
+*/
+
+#if defined(__Bitrig__)
+
+# if defined(__x86_64__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "x86_64-bitrig"
+# elif defined(__arm__) || defined(__thumb__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "arm-bitrig"
+# else
+#  error Unported platform.
+# endif
+
+# include "uconfig.h"
+# undef HAS_STANDARD_IOB
+# define HAS_BSD_IOB
+# undef UNIX_FIND_STACK_BOUNDS
+# define PTHREAD_STACKSEG_FIND_STACK_BOUNDS
+
+/* Default UNIX_STACK_MAXIMUM is too big for a non-root user. */
+# undef UNIX_STACK_MAXIMUM
+# define UNIX_STACK_MAXIMUM 4194304
+
+# define UNDERSCORE_DYNLOAD_SYMBOL_PREFIX
+
+# define USE_DLOPEN_GLOBAL_BY_DEFAULT
+
+# define USE_IEEE_FP_PREDS
+
+# define USE_DYNAMIC_FDSET_SIZE
+
+# define SIGSET_IS_SIGNAL
+
+# define USE_TM_GMTOFF_FIELD
+# define USE_TM_ZONE_FIELD
+
+# define USE_MAP_ANON
+
+# if defined(__x86_64__)
+#  define MZ_USE_JIT_X86_64
+#  define MZ_JIT_USE_MPROTECT
+#  define MZ_TRY_EXTFLONUMS
+# elif defined(__arm__)
+#  define MZ_USE_JIT_ARM
+#  define FFI_CALLBACK_NEED_INT_CLEAR
+# endif
+
+# define FLAGS_ALREADY_SET
+
+#endif
+
+  /************** FreeBSD with gcc ****************/
 
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
- 
+
 # if defined(__i386__)
 #  define SCHEME_PLATFORM_LIBRARY_SUBPATH "i386-freebsd"
 #  define REGISTER_POOR_MACHINE
@@ -396,6 +494,7 @@
 
 # define USE_TM_GMTOFF_FIELD
 # define USE_TM_ZONE_FIELD
+# define MAX_VALID_DATE_SECONDS_BITS 51
 
 # define MZ_JIT_USE_MPROTECT
 
@@ -406,7 +505,7 @@
   /************** SGI/IRIX with SGI cc ****************/
 
 #if  (defined(mips) || defined(__mips)) \
-     && !(defined(ultrix) || defined(__ultrix) || defined(linux))
+     && !(defined(ultrix) || defined(__ultrix) || defined(linux) || defined(__OpenBSD__))
 
 # define SCHEME_PLATFORM_LIBRARY_SUBPATH "mips-irix"
 
@@ -450,7 +549,8 @@
 
   /************** ALPHA/OSF1 with gcc ****************/
 
-# if (defined(__alpha) || defined(__alpha__)) && !defined(linux) && !defined(__NetBSD__)
+# if (defined(__alpha) || defined(__alpha__)) \
+    && !defined(linux) && !defined(__NetBSD__) && !defined(__OpenBSD__)
 
 # define SCHEME_PLATFORM_LIBRARY_SUBPATH "alpha-osf1"
 
@@ -538,19 +638,15 @@
 # if defined(_MSC_VER) || defined(__MINGW32__)
 #  define NO_READDIR
 #  define USE_FINDFIRST
-#  define NO_READLINK
 #  define MKDIR_NO_MODE_FLAG
 # endif
 # if defined(__BORLANDC__)
 #  define DIRENT_NO_NAMLEN
-#  define NO_READLINK
 #  define MKDIR_NO_MODE_FLAG
 # endif
 
 # define TIME_SYNTAX
-# define USE_FTIME
-# define USE_TIMEZONE_VAR_W_DLS
-# define USE_TZNAME_VAR
+# define USE_WIN32_TIME
 # define WINDOWS_GET_PROCESS_TIMES
 # define GETENV_FUNCTION
 # define DIR_FUNCTION
@@ -560,14 +656,8 @@
 
 # define DO_STACK_CHECK
 # define WINDOWS_FIND_STACK_BOUNDS
-/* Default stack size is 1MB, but we try to read
-   the actual size from the executable on startup: */
-# define WINDOWS_DEFAULT_STACK_SIZE 1048576
-# ifdef _WIN64
-#  define STACK_SAFETY_MARGIN 100000
-# endif
 
-# ifndef _WIN64
+# if !defined(_WIN64) || (_MSC_VER >= 1600)
 #  define USE_MZ_SETJMP
 # endif
 
@@ -705,10 +795,6 @@
 
 # define REGISTER_POOR_MACHINE
 
-# ifndef AS_MSVC_EXTENSION
-#  define LINK_EXTENSIONS_BY_TABLE
-# endif
-
 # define MZ_USE_JIT_I386
 
 # define FLAGS_ALREADY_SET
@@ -719,27 +805,25 @@
 
 # if defined(OS_X) || defined(XONX)
 
-#ifdef XONX 
-# ifdef __POWERPC__
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-darwin"
+# if defined(XONX)
+#  define SPLS_MAC "darwin"
+# elif TARGET_OS_IPHONE
+#  define SPLS_MAC "ios"
 # else
-#  ifdef __x86_64__
-#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "x86_64-darwin"
-#  else
-#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "i386-darwin"
-#  endif
+#  define SPLS_MAC "macosx"
 # endif
-#else
-# ifdef __POWERPC__
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-macosx"
+
+# if defined(__POWERPC__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-"SPLS_MAC
+# elif defined(__arm__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "arm-"SPLS_MAC
+# elif defined(__arm64__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "arm64-"SPLS_MAC
+# elif defined(__x86_64__)
+#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "x86_64-"SPLS_MAC
 # else
-#  ifdef __x86_64__
-#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "x86_64-macosx"
-#  else
-#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "i386-macosx"
-#  endif
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "i386-"SPLS_MAC
 # endif
-#endif
 
 # include "uconfig.h"
 
@@ -784,7 +868,10 @@
 #  define OS_X 1
 #endif
 
-#if defined(__POWERPC__)
+#if defined(__arm__)
+# define MZ_USE_JIT_ARM
+#elif defined(__arm64__)
+#elif defined(__POWERPC__)
 # define MZ_USE_JIT_PPC
 #else
 # if defined(__x86_64__)
@@ -831,151 +918,6 @@
 
 # endif
 
-  /************ Macintosh with CodeWarrior (not OS X) *************/
-
-#if ((defined(__MWERKS__) && !defined(__BEOS__) && !defined(__palmos__) && !defined(OS_X))  \
-     || defined(MPW_C) || defined(MPW_CPLUS))
-
-# if defined(__POWERPC__)
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-mac"
-# else
-#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "68k-mac"
-# endif
-
-# define SYSTEM_TYPE_NAME "macos"
-# define MAC_FILE_SYSTEM
-
-# define NO_READDIR
-# define NO_READLINK
-# define USE_MAC_FILE_TOOLBOX
-
-# define MACINTOSH_EVENTS
-# define MAC_MZ_GUI_ENABLED
-# define MAC_CLASSIC_PROCESS_CONTROL
-//# define NEVER_EVER_SLEEP
-# define MACINTOSH_GIVE_TIME
-# define MACINTOSH_SIOUX
-
-# if !defined(__POWERPC__)
-#  define MACINTOSH_SET_STACK
-#  define COMPUTE_NEG_INEXACT_TO_EXACT_AS_POS
-#  define NAN_LT_COMPARISON_WRONG
-#  define SQRT_NAN_IS_WRONG
-#  define ATAN2_DOESNT_WORK_WITH_INFINITIES
-# else
-#  define CODEFRAGMENT_DYNAMIC_LOAD
-# endif
-
-# ifndef MZSCHEME_IS_CODEFRAGMENT
-#  define LINK_EXTENSIONS_BY_TABLE
-# endif
-
-# define DO_STACK_CHECK
-# define MACOS_FIND_STACK_BOUNDS
-# define STACK_SAFETY_MARGIN 10000
-
-# define TIME_SYNTAX
-# define USE_MACTIME
-# define DIR_FUNCTION
-# define TIME_TYPE_IS_UNSIGNED
-
-# define NO_SYS_INCLUDE_SUBDIR
-# define NO_SLEEP
-# define NO_USLEEP
-# define NO_STAT_PROC
-# define DONT_IGNORE_PIPE_SIGNAL
-
-# define TRIG_ZERO_NEEDS_SIGN_CHECK
-
-# define MACOS_UNICODE_SUPPORT
-
-# define USE_MAC_TCP
-
-# define SIGSET_IS_SIGNAL
-
-# ifdef MPW_C
-#  define NO_INLINE_KEYWORD
-# endif
-
-# define FLAGS_ALREADY_SET
-
-#endif
-
-  /************** DOS with Borland C++ ****************/
-  /*          (Never successfully supported)          */
-
-#if defined(__BORLANDC__) && defined(__MSDOS__)
-
-# define SCHEME_PLATFORM_LIBRARY_SUBPATH "dos\\i386"
-
-# define USE_SENORA_GC
-# define DOS_FAR_POINTERS
-# define SMALL_HASH_TABLES
-
-# define SYSTEM_TYPE_NAME "dos"
-# define DOS_FILE_SYSTEM
-# define USE_GETDISK
-# define DIRENT_NO_NAMLEN
-# define NO_READLINK
-# define MKDIR_NO_MODE_FLAG
-
-# define TIME_SYNTAX
-# define USE_FTIME
-# define GETENV_FUNCTION
-# define DIR_FUNCTION
-
-# define DO_STACK_CHECK
-# define USE_STACKAVAIL
-# define STACK_SAFETY_MARGIN 15000
-
-# define IGNORE_BY_CONTROL_87
-
-# define DIR_INCLUDE
-# define IO_INCLUDE
-# define NO_SLEEP
-# define DONT_IGNORE_PIPE_SIGNAL
-
-# define REGISTER_POOR_MACHINE
-
-# define FLAGS_ALREADY_SET
-
-#endif
-
-  /************ PalmOS *************/
-
-#if defined(__palmos__)
-
-# define SCHEME_PLATFORM_LIBRARY_SUBPATH "68k-palm"
-
-# define SYSTEM_TYPE_NAME "palm"
-
-# define NO_FILE_SYSTEM_UTILS
-# define NO_TCP_SUPPORT
-# define MZSCHEME_SOMETHING_OMITTED
-
-# define PALMOS_STUFF
-# define NO_STAT_PROC
-# define NO_USER_BREAK_HANDLER
-# define NO_USLEEP
-
-# define DO_STACK_CHECK
-# define PALM_FIND_STACK_BOUNDS
-# define STACK_SAFETY_MARGIN 1000
-
-# define TIME_SYNTAX
-# define USE_PALMTIME
-# define USER_TIME_IS_CLOCK
-# define TIME_TYPE_IS_UNSIGNED
-
-# define DONT_IGNORE_PIPE_SIGNAL
-# define DONT_IGNORE_FPE_SIGNAL
-
-# define USE_PALM_INF_TESTS
-
-# define FLAGS_ALREADY_SET
-
-#endif
-
   /************ QNX *************/
 
 #if defined(__QNX__)
@@ -1014,7 +956,7 @@
   /************ Dragonfly *************/
 
 #if defined(__DragonFly__)
- 
+
 # if defined(__i386__)
 #  define SCHEME_PLATFORM_LIBRARY_SUBPATH "i386-dragonfly"
 #  define REGISTER_POOR_MACHINE
@@ -1054,6 +996,7 @@
 
 # define USE_TM_GMTOFF_FIELD
 # define USE_TM_ZONE_FIELD
+# define MAX_VALID_DATE_SECONDS_BITS 51
 
 # define MZ_JIT_USE_MPROTECT
 
@@ -1084,7 +1027,7 @@
   /* SYSTEM_TYPE_NAME must be a string; this will be converted into
      a symbol for the result of (system-type) */
 
-  /* SCHEME_PLATFORM_LIBRARY_SUBPATH must be a string; if it is 
+  /* SCHEME_PLATFORM_LIBRARY_SUBPATH must be a string; if it is
      undefined, it is automatically generated into a file named
      "schsys.h" into the same directory as .o files and #included
      by string.c. This string is returned by (system-library-subpath) */
@@ -1101,17 +1044,22 @@
      it defaults to 1000000 */
 
  /* USE_FTIME uses ftime instead of gettimeofday; only for TIME_SYNTAX */
- 
+
  /* USE_PLAIN_TIME uses time; only for TIME_SYNTAX */
- 
+
  /* USE_MACTIME uses the Mac toolbox to implement time functions. */
 
+ /* USE_WIN32_TIME uses the Win32 API to implement time functions. */
+
  /* CLOCK_IS_USER_TIME uses the system time for user milliseconds. */
 
  /* USER_TIME_IS_CLOCK uses the user time for system milliseconds. */
 
  /* TIME_TYPE_IS_UNSIGNED converts time_t values as unsigned. */
 
+ /* MAX_VALID_DATE_SECONDS_BITS sets a maximum number of bits for
+    seconds to pass to localtime() ro gmtime(). */
+
  /* MIN_VALID_DATE_SECONDS sets a minimum vald time in seconds. */
 
  /* PROCESS_FUNCTION adds (process ...) and (system ...) functions */
@@ -1122,7 +1070,7 @@
 
  /* USE_TIMEZONE_VAR gets timezone offset from a timezone global.
     USE_TOD_FOR_TIMEZONE gets timezone offset via gettimeofday.
-    USE_TIMEZONE_VAR_W_DLS is similar, but adds 1 hour when daylight 
+    USE_TIMEZONE_VAR_W_DLS is similar, but adds 1 hour when daylight
      savings is in effect.
     USE_TIMEZONE_AND_ALTZONE_VAR is similar, but uses altzone when
      daylight savings is in effect.
@@ -1132,23 +1080,22 @@
  /* USE_TZNAME_VAR gets the timezone name from a tzname global.
     USE_TM_ZONE_FIELD gets the timezone name from a tm_zone field
      of the tm struct. */
- 
 
   /*******************/
  /*   Filesystem    */
 /*******************/
 
  /* UNIX_FILE_SYSTEM indicates that filenames are as in Unix, with
-    forward slash separators, ".." as the parent directory, "/" 
+    forward slash separators, ".." as the parent directory, "/"
     as the root directory, and case-sensitivity */
 
- /* DOS_FILE_SYSTEM indicates that filenames are as in DOS, with 
-    slash or backward slash separators, ".." as the parent directory, 
+ /* DOS_FILE_SYSTEM indicates that filenames are as in DOS, with
+    slash or backward slash separators, ".." as the parent directory,
     "X:\", "X:/", "\", or "/" as a root directory (for some letter X),
     and case insensitivity */
 
  /* MAC_FILE_SYSTEM indicates that filenames are as on the Macintosh,
-    with colon separators, "" as the parent directory, a volume name 
+    with colon separators, "" as the parent directory, a volume name
     (followed by a colon) as a root directory, and case insensitivity. */
 
  /* EXPAND_FILENAME_TILDE expands ~ in a filename with a user's home
@@ -1158,8 +1105,6 @@
 
  /* NO_MKDIR means that there is no mkdir() function. */
 
- /* NO_READLINK means that there is no readlink() function. */
-
  /* BROKEN_READLINK_NUL_TERMINATOR means that readlink() may
     report a length that includes trailing NUL terminators,
     which should be stripped away. */
@@ -1173,7 +1118,7 @@
  /* DIRENT_NO_NAMLEN specifies that dirent entries do not have a
      d_namlen field; this is used only when NO_READDIR is not
      specified. */
-     
+
  /* MKDIR_NO_MODE_FLAG specifies that mkdir() takes only one argument,
      instead of a directory name and mode flags. */
 
@@ -1214,6 +1159,8 @@
     should be closed after performing a fork() for `process'
     and `system' calls. */
 
+ /* SUBPROCESS_USE_FORK1 uses fork1() instead of fork(). */
+
  /* USE_UNIX_SOCKETS_TCP means that the tcp- procedures can be implemented
     with the standard Unix socket functions. */
 
@@ -1257,10 +1204,10 @@
     an AF_UNSPEC address to disconnect a UDP socket. */
 
  /* UDP_DISCONNECT_EADRNOTAVAIL_OK means that a disconnecting call
-    to connect() might return EADDRNOTAVAIL instead of 
+    to connect() might return EADDRNOTAVAIL instead of
     EAFNOSUPPORT. */
 
- /* MZ_BINARY is combinaed with other flags in all calls to open();
+ /* MZ_BINARY is combined with other flags in all calls to open();
     it can be defined to O_BINARY in Cygwin, for example. */
 
  /* MZ_TCP_LISTEN_IPV6_ONLY_SOCKOPT uses IPV6_V6ONLY for IPv6
@@ -1275,7 +1222,6 @@
     when creating the listener (but only where there is at least once
     IPv4 address). */
 
-
   /***********************/
  /* Threads & Signals  */
 /***********************/
@@ -1297,7 +1243,7 @@
  /* USE_PTHREAD_THREAD_TIMER uses a background pthread to implement
     tread pre-emption. */
 
- /* SIGSET_IS_SIGNAL uses signal() in place of sigset() for Unix. This 
+ /* SIGSET_IS_SIGNAL uses signal() in place of sigset() for Unix. This
     flag is often paired with SIGSET_NEEDS_REINSTALL for traditional
     Unix systems. */
 
@@ -1306,7 +1252,7 @@
     (when this flags is not defined) is that a signal handler is NOT
     reset to SIG_DFL after a handler is called to handle a signal. */
 
- /* DONT_IGNORE_FPE_SIGNAL stops Racket from ignoring floating-point 
+ /* DONT_IGNORE_FPE_SIGNAL stops Racket from ignoring floating-point
     exception signals. */
 
  /* DONT_IGNORE_PIPE_SIGNAL stops Racket from ignoring SIGPIPE
@@ -1328,6 +1274,9 @@
  /* Inexact Arithmetic */
 /**********************/
 
+ /* FLOATING_POINT_IS_NOT_IEEE disables inexact->exact conversion via
+    parsing of IEEE-format bits. */
+
  /* USE_SINGLE_FLOATS turns on support for single-precision
     floating point numbers. Otherwise, floating point numbers
     are always represented in double-precision. */
@@ -1362,9 +1311,9 @@
     converts (a == a) to TRUE, even if `a' is floating-point. Used
     only when USE_[SCO_]IEEE_FP_PREDS is not defined. */
 
- /* C_COMPILER_USES_SSE indicates that the C compiler generates SSE
+ /* C_COMPILER_USES_SSE indicates that the C compiler generates SSE2
     instructions for `double' arithmetic. This flag is turned on
-    automatically if __SSE_MATH__ is defined (usually by gcc). */
+    automatically if __SSE2_MATH__ is defined (usually by gcc). */
 
  /* MZ_LONG_DOUBLE enables extflonum support. */
 
@@ -1411,7 +1360,7 @@
  /* NAN_EQUALS_ANYTHING indicates that the compiler is broken and
     equality comparisons with +nan.0 always return #t. Currently
     used for MSVC++ */
-    
+
  /* ZERO_MINUS_ZERO_IS_POS_ZERO indicates that something (compiler?
     machine? fp flags?) is broken so that 0.0 - 0.0 = 0.0 instead of
     -0.0. This flag doesn't fix Racket completely, since (- 0.0) is
@@ -1422,22 +1371,22 @@
     by < or <=. Probably the compiler implements < as !>. */
 
  /* USE_EXPLICT_FP_FORM_CHECK circumvents bugs in strtod() under Linux,
-    SunOS/Solaris, and HP/UX by explicit pre-checking the form of the 
+    SunOS/Solaris, and HP/UX by explicit pre-checking the form of the
     number and looking for values that are obviously +inf.0 or -inf.0 */
 
  /* POW_HANDLES_CASES_CORRECTLY indicates that the pow() library procedure
     handles all +/-inf.0, +/-0.0, or +nan.0 cases according to C99. This
     might save time on redundant checking before Racket calls pow(). */
-    
+
  /* ATAN2_DOESNT_WORK_WITH_INFINITIES indicates that atan2(+/-inf, +/-inf)
-    is not the same as atan2(1, 1). */ 
-    
+    is not the same as atan2(1, 1). */
+
  /* ATAN2_DOESNT_WORK_WITH_NAN indicates that atan2(+nan.0, _) and
-    atan2(_, +nan.0) don't produce +nan.0. */ 
-    
+    atan2(_, +nan.0) don't produce +nan.0. */
+
  /* SQRT_NAN_IS_WRONG indicates that (sqrt +nan.0) must be forced to +nan.0
     (i.e., the C library function is bad). */
-    
+
  /* COMPUTE_NEG_INEXACT_TO_EXACT_AS_POS computes inexact->exact of some
     negative inexact number x by computing the result for -x and negating
     it. Use this if (inexact->exact -0.1) is wrong. */
@@ -1469,7 +1418,7 @@
  /* NO_LONG_LONG_TYPE indicates that long long is not supported */
 
   /***********************/
- /* Stack Maniuplations */
+ /* Stack Manipulations */
 /***********************/
 
  /* DO_STACK_CHECK checks for stack overflow during execution.
@@ -1490,6 +1439,8 @@
      line.
     ASSUME_FIXED_STACK_SIZE assumes that the main stack size is
      always FIXED_STACK_SIZE.
+    PTHREAD_STACKSEG_FIND_STACK_BOUNDS finds stack bounds using
+     pthread_stackseg_np().
     Use only one of these if DO_STACK_CHECK is used, or none otherwise. */
 
  /* FIXED_STACK_SIZE <X> sets the stack size to <X> when the
@@ -1497,7 +1448,8 @@
 
  /* STACK_SAFETY_MARGIN <X> sets the number of bytes that should be
      available on the stack for "safety" to <X>. Used only if
-     DO_STACK_CHECK is used. STACK_SAFETY_MARGIN defaults to 50000. */
+     DO_STACK_CHECK is used. STACK_SAFETY_MARGIN defaults to 50000
+     for a 32-bit platform, twice as much for a 64-bit platform. */
 
  /* UNIX_LIMIT_STACK <X> limits stack usage to <X> bytes. This may
      be necessary to avoid GC-setup traversal over too much memory
@@ -1511,9 +1463,9 @@
 /***********************/
 
  /* UNIX_DYNAMIC_LOAD implements dynamic extensions under Unix
-     using dlopen(); you may have to add the -ldl flag in the LIBS 
-     Makefile variable. The library doesn't exist under Linux without 
-     ELF, so it won't work. If you get linker errors about dlopen(), etc., 
+     using dlopen(); you may have to add the -ldl flag in the LIBS
+     Makefile variable. The library doesn't exist under Linux without
+     ELF, so it won't work. If you get linker errors about dlopen(), etc.,
      this flag and the -ldl linker flag are the things to adjust.
     SHL_DYNAMIC_LOAD implement HP/UX dynamic loading.
     WINDOWS_DYNAMIC_LOAD implements dynamic extensions under Windows
@@ -1523,7 +1475,7 @@
     Use only one or none of these. */
 
  /* UNDERSCORE_DYNLOAD_SYMBOL_PREFIX with UNIX_DYNAMIC_LOAD means that
-    an extra underscore ("_") must be placed in front of the name passed 
+    an extra underscore ("_") must be placed in front of the name passed
     to dlopen(). */
 
  /* USE_DLOPEN_GLOBAL_BY_DEFAULT opens shared libraries in "global"
@@ -1557,10 +1509,10 @@
  /*     Heap Images     */
 /***********************/
 
- /* UNIX_IMAGE_DUMPS turns on image save and restore for Unix systems. 
-     This will only work if the final application is statically linked. 
-     (As an exception, the dynamic-linking library itself can be 
-     dynamically linked. This works because loading an extension in 
+ /* UNIX_IMAGE_DUMPS turns on image save and restore for Unix systems.
+     This will only work if the final application is statically linked.
+     (As an exception, the dynamic-linking library itself can be
+     dynamically linked. This works because loading an extension in
      Racket automatically turns off image saving.) */
 
   /*****************************/
@@ -1568,16 +1520,16 @@
 /*****************************/
 
  /* MACINTOSH_EVENTS enables specific support for Mac Classic */
- 
- /* MAC_MZ_GUI_ENABLED activates the windowed I/O code (use under classic) 
+
+ /* MAC_MZ_GUI_ENABLED activates the windowed I/O code (use under classic)
      (should maybe use MACINTOSH_SIOUX instead?) */
 
- /* MAC_CLASSIC_PROCESS_CONTROL swaps the UNIX process commands for the 
+ /* MAC_CLASSIC_PROCESS_CONTROL swaps the UNIX process commands for the
     mac family (use under classic) */
 
  /* OS_X enables specific support for Mac OS X, e.g. the location of the
     prefs directory */
- 
+
  /* MACINTOSH_GIVE_TIME lets background processes run when checking for
      a user break */
 
@@ -1590,9 +1542,14 @@
  /*    Miscellaneous    */
 /***********************/
 
- /* USE_MAP_ANON indicates that mmap should use BSD's MAP_ANON flag
+ /* USE_MAP_ANON indicates that mmap() should use BSD's MAP_ANON flag
     rather than trying to open /dev/zero */
 
+ /* PREFER_MMAP_LARGE_BLOCKS indicates that mmap() should be called with
+    large block sizes as much as possible, because the actual allocated
+    size for small requests (on the order of the page size) is much
+    larger than the request. */
+
  /* REGISTER_POOR_MACHINE guides a hand optimization that seems to
     be work best one way for Sparc machines, and better the other
     way for x86 machines. */
@@ -1613,7 +1570,7 @@
  /* NO_INLINE_KEYWORD indicates that the C compiler doesn't recognize
     C's `inline' keyword. */
 
- /* NO_USER_BREAK_HANDLER turns off handling of SIGINT, SIGTERM, and 
+ /* NO_USER_BREAK_HANDLER turns off handling of SIGINT, SIGTERM, and
     SIGHUP in main.c */
 
  /* NO_SIGTERM_HANDLER turns off handling of SIGTERM in main.c */
@@ -1628,10 +1585,10 @@
 
  /* UNISTD_INCLUDE if there's a <unistd.h> file (mainly for Unix). */
 
- /* SELECT_INCLUDE if there's a <sys/select.h> file (mainly for Unix) 
+ /* SELECT_INCLUDE if there's a <sys/select.h> file (mainly for Unix)
      to be used with FILES_HAVE_FDS. */
 
- /* BSTRING_INCLUDE if there's a <bstring.h> file (mainly for Unix) 
+ /* BSTRING_INCLUDE if there's a <bstring.h> file (mainly for Unix)
      to be used with FILES_HAVE_FDS. */
 
  /* NO_SYS_INCLUDE_SUBDIR if include files should all be <XXX.h>; no
@@ -1650,7 +1607,7 @@
  /* NO_SLEEP means that there is no sleep() function. Used only in
     standalone Racket. */
 
- /* NO_USLEEP means that there is no usleep() function. Used only in 
+ /* NO_USLEEP means that there is no usleep() function. Used only in
     standalone Racket. Used only if NO_SLEEP is undefined. */
 
  /* NO_STRERROR_AVAILABLE means that strerror() is not available. */
@@ -1665,6 +1622,10 @@
  /* WIN32S_HACK uses a special hack to implement threads under Win32s
     with some compilers. Obsolete. */
 
+ /* FFI_CALLBACK_NEED_INT_CLEAR indiates thet libffi callback results
+    that are smaller than an `int' should clear `int'-sized space
+    in the result area. */
+
 #endif  /* FLAGS_ALREADY_SET */
 
 /****** (END CONFIGURATION FLAG DESCRPTIONS AND DEFAULTS) *******/
