@@ -731,38 +731,6 @@ Scheme_Object *scheme_register_stx_in_prefix(Scheme_Object *var, Scheme_Comp_Env
   return o;
 }
 
-void scheme_register_unsafe_in_prefix(Scheme_Comp_Env *env, 
-                                      Scheme_Compile_Info *rec, int drec,
-                                      Scheme_Env *menv)
-{
-  Scheme_Object *v, *insp;
-
-  if (rec && rec[drec].dont_mark_local_use) {
-    return;
-  }
-
-  insp = menv->module->insp;
-
-  v = env->prefix->uses_unsafe;
-  if (!v)
-    v = insp;
-  else if (!SAME_OBJ(v, insp)) {
-    Scheme_Hash_Tree *ht;
-
-    if (SCHEME_HASHTRP(v)) {
-      ht = (Scheme_Hash_Tree *)v;
-    } else {
-      ht = scheme_make_hash_tree(0);
-      ht = scheme_hash_tree_set(ht, v, scheme_true);
-    }
-    
-    if (!scheme_hash_tree_get(ht, insp)) {
-      ht = scheme_hash_tree_set(ht, insp, scheme_true);
-      env->prefix->uses_unsafe = (Scheme_Object *)ht;
-    }
-  }
-}
-
 /*========================================================================*/
 /*                     compile-time env, lookup bindings                  */
 /*========================================================================*/
@@ -1291,8 +1259,8 @@ int scheme_tl_id_is_sym_used(Scheme_Hash_Table *marked_names, Scheme_Object *sym
           scheme_hash_set(rev_ht, SCHEME_CDR(a), scheme_true);
         }
       }
-      scheme_hash_set(marked_names, scheme_false, (Scheme_Object *)rev_ht);
     }
+    scheme_hash_set(marked_names, scheme_false, (Scheme_Object *)rev_ht);
   }
 
   if (scheme_hash_get(rev_ht, sym))
@@ -1831,7 +1799,7 @@ scheme_lookup_binding(Scheme_Object *find_id, Scheme_Comp_Env *env, int flags,
 	  if (SCHEME_FALSEP(val)) {
 	    /* Corresponds to a run-time binding (but will be replaced later
 	       through a renaming to a different binding) */
-            if (flags & SCHEME_OUT_OF_CONTEXT_LOCAL)
+            if (flags & (SCHEME_OUT_OF_CONTEXT_LOCAL | SCHEME_SETTING))
               return scheme_make_local(scheme_local_type, 0, 0);
             return NULL;
 	  }
@@ -2386,7 +2354,7 @@ Scheme_Object *scheme_local_lift_require(Scheme_Object *form, Scheme_Object *ori
   mark = scheme_new_mark();
 
   if (SCHEME_RPAIRP(data))
-    form = scheme_parse_lifted_require(form, phase, mark, SCHEME_CAR(data));
+    form = scheme_parse_lifted_require(form, phase, mark, SCHEME_CAR(data), &orig_form);
   else {
     form = scheme_toplevel_require_for_expand(form, phase, env, mark);
     need_prepare = 1;
