@@ -138,6 +138,10 @@ typedef uintptr_t hash_v_t;
 
 #define MAX_HASH_DEPTH 128
 
+/* For detecting and debugging accidental dependencies on hash-table order,
+   it might be helpful to invert the order at the lowest level: */
+/* #define REVERSE_HASH_TABLE_ORDER 1 */
+
 /*========================================================================*/
 /*                         hashing functions                              */
 /*========================================================================*/
@@ -184,6 +188,11 @@ static int not_stx_bound_eq(char *a, char *b)
 /*                         normal hash table                              */
 /*========================================================================*/
 
+#ifdef REVERSE_HASH_TABLE_ORDER
+# define HASH_TO_ARRAY_INDEX(h, mask) ((mask) - (h))
+#else
+# define HASH_TO_ARRAY_INDEX(h, mask) (h)
+#endif
 
 Scheme_Hash_Table *scheme_make_hash_table(int type)
 {
@@ -228,16 +237,22 @@ static Scheme_Object *do_hash(Scheme_Hash_Table *table, Scheme_Object *key, int
   mask = table->size - 1;
 
   if (table->make_hash_indices) {
-    GC_CAN_IGNORE intptr_t *_h2x;
-    if (table->compare) {
+    if (table->compare == scheme_compare_equal) {
       h2 = 0;
-      _h2x = NULL;
-    } else
-      _h2x = &h2x;
-    table->make_hash_indices((void *)key, &hx, _h2x);
-    h = to_unsigned_hash(hx) & mask;
-    if (_h2x)
-      h2 = (to_unsigned_hash(h2x) & mask) | 1;
+      hx = scheme_equal_hash_key(key);
+      h = to_unsigned_hash(hx) & mask;
+    } else {
+      GC_CAN_IGNORE intptr_t *_h2x;
+      if (table->compare) {
+        h2 = 0;
+        _h2x = NULL;
+      } else
+        _h2x = &h2x;
+      table->make_hash_indices((void *)key, &hx, _h2x);
+      h = to_unsigned_hash(hx) & mask;
+      if (_h2x)
+        h2 = (to_unsigned_hash(h2x) & mask) | 1;
+    }
   } else {
     uintptr_t lkey;
     lkey = PTR_TO_LONG((Scheme_Object *)key);
@@ -246,46 +261,75 @@ static Scheme_Object *do_hash(Scheme_Hash_Table *table, Scheme_Object *key, int
   }
 
   keys = table->keys;
-  
+
   if (table->compare) {
-    scheme_hash_request_count++;
-    while ((tkey = keys[h])) {
-      if (SAME_PTR(tkey, GONE)) {
-	if (set > 1) {
-	  useme = h;
-	  set = 1;
-	}
-      } else if (!table->compare(tkey, (char *)key)) {
-	if (set) {
-	  table->vals[h] = val;
-	  if (!val) {
-	    keys[h] = GONE;
-	    --table->count;
-	  }
-	  return val;
-	} else
-	  return table->vals[h];
+    if (table->compare == scheme_compare_equal) {
+      /* Direct calls can be significant faster than indirect */
+      scheme_hash_request_count++;
+      while ((tkey = keys[HASH_TO_ARRAY_INDEX(h, mask)])) {
+        if (SAME_PTR(tkey, GONE)) {
+          if (set > 1) {
+            useme = h;
+            set = 1;
+          }
+        } else if (scheme_equal(tkey, key)) {
+          if (set) {
+            table->vals[HASH_TO_ARRAY_INDEX(h, mask)] = val;
+            if (!val) {
+              keys[HASH_TO_ARRAY_INDEX(h, mask)] = GONE;
+              --table->count;
+            }
+            return val;
+          } else
+            return table->vals[HASH_TO_ARRAY_INDEX(h, mask)];
+        }
+        scheme_hash_iteration_count++;
+        if (!h2) {
+          h2x = scheme_equal_hash_key2(key);
+          h2 = (to_unsigned_hash(h2x) & (table->size - 1)) | 1;
+        }
+        h = (h + h2) & mask;
       }
-      scheme_hash_iteration_count++;
-      if (!h2) {
-        table->make_hash_indices((void *)key, NULL, &h2x);
-        h2 = (to_unsigned_hash(h2x) & (table->size - 1)) | 1;
+    } else {
+      scheme_hash_request_count++;
+      while ((tkey = keys[HASH_TO_ARRAY_INDEX(h, mask)])) {
+        if (SAME_PTR(tkey, GONE)) {
+          if (set > 1) {
+            useme = h;
+            set = 1;
+          }
+        } else if (!table->compare(tkey, (char *)key)) {
+          if (set) {
+            table->vals[HASH_TO_ARRAY_INDEX(h, mask)] = val;
+            if (!val) {
+              keys[HASH_TO_ARRAY_INDEX(h, mask)] = GONE;
+              --table->count;
+            }
+            return val;
+          } else
+            return table->vals[HASH_TO_ARRAY_INDEX(h, mask)];
+        }
+        scheme_hash_iteration_count++;
+        if (!h2) {
+          table->make_hash_indices((void *)key, NULL, &h2x);
+          h2 = (to_unsigned_hash(h2x) & (table->size - 1)) | 1;
+        }
+        h = (h + h2) & mask;
       }
-      h = (h + h2) & mask;
     }
   } else {
     scheme_hash_request_count++;
-    while ((tkey = keys[h])) {
+    while ((tkey = keys[HASH_TO_ARRAY_INDEX(h, mask)])) {
       if (SAME_PTR(tkey, key)) {
 	if (set) {
-	  table->vals[h] = val;
+	  table->vals[HASH_TO_ARRAY_INDEX(h, mask)] = val;
 	  if (!val) {
-	    keys[h] = GONE;
+	    keys[HASH_TO_ARRAY_INDEX(h, mask)] = GONE;
 	    --table->count;
 	  }
 	  return val;
 	} else
-	  return table->vals[h];
+	  return table->vals[HASH_TO_ARRAY_INDEX(h, mask)];
       } else if (SAME_PTR(tkey, GONE)) {
 	if (set > 1) {
 	  useme = h;
@@ -335,8 +379,8 @@ static Scheme_Object *do_hash(Scheme_Hash_Table *table, Scheme_Object *key, int
   }
 
   table->count++;
-  table->keys[h] = key;
-  table->vals[h] = val;
+  table->keys[HASH_TO_ARRAY_INDEX(h, mask)] = key;
+  table->vals[HASH_TO_ARRAY_INDEX(h, mask)] = val;
 
   return val;
 }
@@ -360,11 +404,11 @@ static Scheme_Object *do_hash_set(Scheme_Hash_Table *table, Scheme_Object *key,
   keys = table->keys;
   
   scheme_hash_request_count++;
-  while ((tkey = keys[h])) {
+  while ((tkey = keys[HASH_TO_ARRAY_INDEX(h, mask)])) {
     if (SAME_PTR(tkey, key)) {
-      table->vals[h] = val;
+      table->vals[HASH_TO_ARRAY_INDEX(h, mask)] = val;
       if (!val) {
-	keys[h] = GONE;
+	keys[HASH_TO_ARRAY_INDEX(h, mask)] = GONE;
 	--table->count;
       }
       return val;
@@ -391,8 +435,8 @@ static Scheme_Object *do_hash_set(Scheme_Hash_Table *table, Scheme_Object *key,
   }
 
   table->count++;
-  table->keys[h] = key;
-  table->vals[h] = val;
+  table->keys[HASH_TO_ARRAY_INDEX(h, mask)] = key;
+  table->vals[HASH_TO_ARRAY_INDEX(h, mask)] = val;
 
   return val;
 }
@@ -415,9 +459,9 @@ XFORM_NONGCING static Scheme_Object *do_hash_get(Scheme_Hash_Table *table, Schem
   keys = table->keys;
   
   scheme_hash_request_count++;
-  while ((tkey = keys[h])) {
+  while ((tkey = keys[HASH_TO_ARRAY_INDEX(h, mask)])) {
     if (SAME_PTR(tkey, key)) {
-      return table->vals[h];
+      return table->vals[HASH_TO_ARRAY_INDEX(h, mask)];
     } 
     scheme_hash_iteration_count++;
     h = (h + h2) & mask;
@@ -700,7 +744,7 @@ get_bucket (Scheme_Bucket_Table *table, const char *key, int add, Scheme_Bucket
   if (table->weak) {
     int reuse_bucket = 0;
     scheme_hash_request_count++;
-    while ((bucket = table->buckets[h])) {
+    while ((bucket = table->buckets[HASH_TO_ARRAY_INDEX(h, mask)])) {
       if (bucket->key) {
 	void *hk = (void *)HT_EXTRACT_WEAK(bucket->key);
 	if (!hk) {
@@ -724,7 +768,7 @@ get_bucket (Scheme_Bucket_Table *table, const char *key, int add, Scheme_Bucket
     }
   } else {
     scheme_hash_request_count++;
-    while ((bucket = table->buckets[h])) {
+    while ((bucket = table->buckets[HASH_TO_ARRAY_INDEX(h, mask)])) {
       if (SAME_PTR(bucket->key, key))
 	return bucket;
       else if (compare && !compare((void *)bucket->key, (void *)key))
@@ -760,6 +804,10 @@ get_bucket (Scheme_Bucket_Table *table, const char *key, int add, Scheme_Bucket
       if (actual * FILL_FACTOR < table->count) {
 	/* Decrement size so that the table won't actually grow. */
 	table->size >>= 1;
+        if ((table->size > 64) && (2 * actual * FILL_FACTOR < table->count)) {
+          /* Allow the table to shrink */
+          table->size >>= 1;
+        }
       }
     }
 
@@ -793,7 +841,7 @@ get_bucket (Scheme_Bucket_Table *table, const char *key, int add, Scheme_Bucket
   else
     bucket = allocate_bucket(table, key, NULL);
 
-  table->buckets[h] = bucket;
+  table->buckets[HASH_TO_ARRAY_INDEX(h, mask)] = bucket;
 
   table->count++;
 
@@ -1162,21 +1210,13 @@ XFORM_NONGCING static uintptr_t long_dbl_hash2_val(long_double d)
    http://www.burtleburtle.net/bob/hash/doobs.html: */
 #define MZ_MIX(k) (k += (k << 10), k ^= (k >> 6))
 
-static uintptr_t equal_hash_key(Scheme_Object *o, uintptr_t k, Hash_Info *hi)
+XFORM_NONGCING static uintptr_t fast_equal_hash_key(Scheme_Object *o, uintptr_t k, int *_done)
 {
   Scheme_Type t;
-  Scheme_Object *orig_obj;
 
  top:
-  orig_obj = o;
-  if (SCHEME_CHAPERONEP(o))
-    o = ((Scheme_Chaperone *)o)->val;
-  
   t = SCHEME_TYPE(o);
   k += t;
-
-  if (hi->depth > (MAX_HASH_DEPTH << 1))
-    return k;
   
   switch(t) {
   case scheme_integer_type:
@@ -1213,17 +1253,114 @@ static uintptr_t equal_hash_key(Scheme_Object *o, uintptr_t k, Hash_Info *hi)
     break;
   case scheme_rational_type:
     {
-      k += equal_hash_key(scheme_rational_numerator(o), 0, hi);
+      k += fast_equal_hash_key(scheme_rational_numerator(o), 0, _done);
       o = scheme_rational_denominator(o);
       break;
     }
   case scheme_complex_type:
     {
       Scheme_Complex *c = (Scheme_Complex *)o;
-      k += equal_hash_key(c->r, 0, hi);
+      k += fast_equal_hash_key(c->r, 0, _done);
       o = c->i;
       break;
     }
+  case scheme_cpointer_type:
+    {
+      k = (k << 3) + k;
+      k += (uintptr_t)((char *)SCHEME_CPTR_VAL(o) + SCHEME_CPTR_OFFSET(o));
+      return k;
+    }
+  case scheme_char_type:
+    return k + SCHEME_CHAR_VAL(o);
+  case scheme_byte_string_type:
+  case scheme_unix_path_type:
+  case scheme_windows_path_type:
+    {
+      int i = SCHEME_BYTE_STRLEN_VAL(o);
+      char *s = SCHEME_BYTE_STR_VAL(o);
+      
+      while (i--) {
+	k += s[i];
+        MZ_MIX(k);
+      }
+      
+      return k;
+    }
+  case scheme_char_string_type:
+    {
+      int i = SCHEME_CHAR_STRLEN_VAL(o);
+      mzchar *s = SCHEME_CHAR_STR_VAL(o);
+      
+      while (i--) {
+	k += s[i];
+        MZ_MIX(k);
+      }
+      
+      return k;
+    }
+  case scheme_regexp_type:
+    {
+      o = scheme_regexp_source(o);
+      break;
+    }
+# ifndef MZ_PRECISE_GC
+  case scheme_keyword_type:
+  case scheme_symbol_type:
+    {
+      Scheme_Symbol *s = (Scheme_Symbol *)o;
+      if (!(MZ_OPT_HASH_KEY(&s->iso) & 0x1)) {
+	/* Interned. Make key depend only on the content. */
+	if (!(MZ_OPT_HASH_KEY(&s->iso) & 0xFFFC)) {
+	  int i, h = 0;
+	  for (i = s->len; i--; ) {
+	    h += (h << 5) + h + s->s[i];
+	  }
+	  h += (h << 2);
+	  if (!(((short)h) & 0xFFFC))
+	    h = 0x10;
+	  MZ_OPT_HASH_KEY(&s->iso) |= (((short)h) & 0xFFFC);
+	}
+	
+	return k + (MZ_OPT_HASH_KEY(&s->iso) & 0xFFFC);
+      } else
+	return k + PTR_TO_LONG(o);
+    }
+# endif
+  default:
+    {
+      *_done = 0;
+      return 0;
+    }
+  }
+
+  MZ_MIX(k);
+  goto top;
+}
+
+static uintptr_t equal_hash_key(Scheme_Object *o, uintptr_t k, Hash_Info *hi)
+{
+  Scheme_Type t;
+  Scheme_Object *orig_obj;
+  int done;
+  uintptr_t k2;
+
+ top:
+  orig_obj = o;
+  if (SCHEME_CHAPERONEP(o))
+    o = ((Scheme_Chaperone *)o)->val;
+  
+  t = SCHEME_TYPE(o);
+  k += t;
+
+  if (hi->depth > (MAX_HASH_DEPTH << 1))
+    return k;
+
+  done = 1;
+  k2 = fast_equal_hash_key(o, k, &done);
+  if (done)
+    return k2;
+  
+  switch(t) {
   case scheme_pair_type:
     {
 #     include "mzhashchk.inc"
@@ -1245,12 +1382,6 @@ static uintptr_t equal_hash_key(Scheme_Object *o, uintptr_t k, Hash_Info *hi)
       o = SCHEME_CDR(o);
       break;
     }
-  case scheme_cpointer_type:
-    {
-      k = (k << 3) + k;
-      k += (uintptr_t)((char *)SCHEME_CPTR_VAL(o) + SCHEME_CPTR_OFFSET(o));
-      return k;
-    }
   case scheme_vector_type:
   case scheme_fxvector_type:
   case scheme_wrap_chunk_type:
@@ -1315,39 +1446,6 @@ static uintptr_t equal_hash_key(Scheme_Object *o, uintptr_t k, Hash_Info *hi)
       return k;
     }
 #endif
-  case scheme_char_type:
-    return k + SCHEME_CHAR_VAL(o);
-  case scheme_byte_string_type:
-  case scheme_unix_path_type:
-  case scheme_windows_path_type:
-    {
-      int i = SCHEME_BYTE_STRLEN_VAL(o);
-      char *s = SCHEME_BYTE_STR_VAL(o);
-      
-      while (i--) {
-	k += s[i];
-        MZ_MIX(k);
-      }
-      
-      return k;
-    }
-  case scheme_char_string_type:
-    {
-      int i = SCHEME_CHAR_STRLEN_VAL(o);
-      mzchar *s = SCHEME_CHAR_STR_VAL(o);
-      
-      while (i--) {
-	k += s[i];
-        MZ_MIX(k);
-      }
-      
-      return k;
-    }
-  case scheme_regexp_type:
-    {
-      o = scheme_regexp_source(o);
-      break;
-    }
   case scheme_structure_type:
   case scheme_proc_struct_type:
     {
@@ -1544,29 +1642,14 @@ static uintptr_t equal_hash_key(Scheme_Object *o, uintptr_t k, Hash_Info *hi)
       
       return k;
     }
-# ifndef MZ_PRECISE_GC
-  case scheme_keyword_type:
-  case scheme_symbol_type:
+  case scheme_place_bi_channel_type:
     {
-      Scheme_Symbol *s = (Scheme_Symbol *)o;
-      if (!(MZ_OPT_HASH_KEY(&s->iso) & 0x1)) {
-	/* Interned. Make key depend only on the content. */
-	if (!(MZ_OPT_HASH_KEY(&s->iso) & 0xFFFC)) {
-	  int i, h = 0;
-	  for (i = s->len; i--; ) {
-	    h += (h << 5) + h + s->s[i];
-	  }
-	  h += (h << 2);
-	  if (!(((short)h) & 0xFFFC))
-	    h = 0x10;
-	  MZ_OPT_HASH_KEY(&s->iso) |= (((short)h) & 0xFFFC);
-	}
-	
-	return k + (MZ_OPT_HASH_KEY(&s->iso) & 0xFFFC);
-      } else
-	return k + PTR_TO_LONG(o);
+      k += 7;
+      /* a bi channel has sendch and recvch, but
+         sends are the same iff recvs are the same: */
+      o = (Scheme_Object *)((Scheme_Place_Bi_Channel *)o)->link->sendch;
     }
-# endif
+    break;
   case scheme_resolved_module_path_type:
     /* Needed for interning */
     {
@@ -1586,15 +1669,7 @@ static uintptr_t equal_hash_key(Scheme_Object *o, uintptr_t k, Hash_Info *hi)
       break;
     }
     break;
-  case scheme_place_bi_channel_type:
-    {
-      k += 7;
-      /* a bi channel has sendch and recvch, but
-         sends are the same iff recvs are the same: */
-      o = (Scheme_Object *)((Scheme_Place_Bi_Channel *)o)->link->sendch;
-    }
-    break;
-  default:    
+  default:
     {
       Scheme_Primary_Hash_Proc h1 = scheme_type_hash1s[t];
       if (h1)
@@ -1608,7 +1683,7 @@ static uintptr_t equal_hash_key(Scheme_Object *o, uintptr_t k, Hash_Info *hi)
   goto top;
 }
 
-intptr_t scheme_equal_hash_key(Scheme_Object *o)
+static intptr_t slow_equal_hash_key(Scheme_Object *o)
 {
   Hash_Info hi;
 
@@ -1619,6 +1694,18 @@ intptr_t scheme_equal_hash_key(Scheme_Object *o)
   return to_signed_hash(equal_hash_key(o, 0, &hi));
 }
 
+intptr_t scheme_equal_hash_key(Scheme_Object *o)
+{
+  uintptr_t k;
+  int done = 1;
+
+  k = fast_equal_hash_key(o, 0, &done);
+  if (done)
+    return to_signed_hash(k);
+  else
+    return slow_equal_hash_key(o);
+}
+
 intptr_t scheme_equal_hash_key2(Scheme_Object *o)
 {
   Hash_Info hi;
@@ -2096,6 +2183,18 @@ typedef struct AVLNode {
 # define AVL_ASSERT_ONLY(x) /* empty */
 #endif
 
+#ifdef REVERSE_HASH_TABLE_ORDER
+# define HASH_KEY_GT_OP <
+# define HASH_KEY_LT_OP >
+# define QUICK_TABLE_INIT_LEFT  1
+# define QUICK_TABLE_INIT_RIGHT 0
+#else
+# define HASH_KEY_GT_OP >
+# define HASH_KEY_LT_OP <
+# define QUICK_TABLE_INIT_LEFT  0
+# define QUICK_TABLE_INIT_RIGHT 1
+#endif
+
 XFORM_NONGCING static int get_height(AVLNode* t)
 {
   if (t == NULL)
@@ -2153,7 +2252,7 @@ XFORM_NONGCING static AVLNode *avl_find(uintptr_t code, AVLNode *s)
     
     if (s->code == code)
       return s;
-    else if (s->code > code)
+    else if (s->code HASH_KEY_GT_OP code)
       s = s->left;
     else
       s = s->right;
@@ -2241,7 +2340,7 @@ static AVLNode *avl_ins(uintptr_t code, Scheme_Object *key, Scheme_Object *val,
   if (t == NULL)
     return AVL_CHK(make_avl(NULL, code, key, val, NULL), code);
   else {
-    if (t->code > code) {
+    if (t->code HASH_KEY_GT_OP code) {
       /* insert on left */
       AVLNode *left;
 
@@ -2254,7 +2353,7 @@ static AVLNode *avl_ins(uintptr_t code, Scheme_Object *key, Scheme_Object *val,
       fix_height(t);
       
       return check_rotate_right(t);
-    } else if (t->code < code) {
+    } else if (t->code HASH_KEY_LT_OP code) {
       /* insert on right */
       AVLNode *right;
       
@@ -2277,7 +2376,7 @@ static AVLNode* avl_del(AVLNode* t, uintptr_t code)
   if (t == NULL)
     return NULL;
   else {
-    if (code < t->code) {
+    if (code HASH_KEY_LT_OP t->code) {
       /* delete on left */
       AVLNode *new_left;
       
@@ -2289,7 +2388,7 @@ static AVLNode* avl_del(AVLNode* t, uintptr_t code)
       t->left = new_left;
       fix_height(t);
       return check_rotate_left(t);
-    } else if (code > t->code) {
+    } else if (code HASH_KEY_GT_OP t->code) {
       /* delete on right */
       AVLNode *new_right;
       
@@ -2337,7 +2436,7 @@ static AVLNode *avl_replace(AVLNode *s, AVLNode *orig, AVLNode *naya)
 
   s = avl_clone(s);
 
-  if (s->code > orig->code) {
+  if (s->code HASH_KEY_GT_OP orig->code) {
     next = avl_replace(s->left, orig, naya);
     s->left = next;
   } else {
@@ -2520,8 +2619,8 @@ static void *hash_tree_set(Scheme_Hash_Tree *tree, Scheme_Object *key, Scheme_Ob
         static AVLNode *sn;
 
         /* avoid intermediate allocations by constructing directly: */
-        sn = make_avl(NULL, 1, added->key, added->val, NULL);
-        sn = make_avl(NULL, 0, key, val, sn);
+        sn = make_avl(NULL, QUICK_TABLE_INIT_RIGHT, added->key, added->val, NULL);
+        sn = make_avl(NULL, QUICK_TABLE_INIT_LEFT, key, val, sn);
 
         val = (Scheme_Object *)sn;
         key = NULL;
