@@ -91,8 +91,8 @@ void **GC_variable_stack;
 #endif
 
 #ifndef MZ_PRECISE_GC
-extern MZ_DLLIMPORT void GC_register_late_disappearing_link(void **link, void *obj);
-extern MZ_DLLIMPORT void GC_register_indirect_disappearing_link(void **link, void *obj);
+extern MZGC_DLLIMPORT void GC_register_late_disappearing_link(void **link, void *obj);
+extern MZGC_DLLIMPORT void GC_register_indirect_disappearing_link(void **link, void *obj);
 #endif
 
 SHARED_OK static int use_registered_statics;
@@ -102,7 +102,7 @@ SHARED_OK static int use_registered_statics;
 /************************************************************************/
 
 #if !defined(MZ_PRECISE_GC) && !defined(USE_SENORA_GC)
-extern MZ_DLLIMPORT void GC_init();
+extern MZGC_DLLIMPORT void GC_init();
 #endif
 
 void scheme_set_stack_base(void *base, int no_auto_statics) XFORM_SKIP_PROC
@@ -242,7 +242,7 @@ void scheme_setup_thread_local_key_if_needed() XFORM_SKIP_PROC
     abort();
   }
 # if defined(__APPLE__) && defined(__MACH__)
-  /* Darwin version 11 (Max OS X Lion) changes the offset from %gs
+  /* Darwin version 11 (Mac OS X Lion) changes the offset from %gs
      for thread-local storage. */
   {
     int name[2];
@@ -1271,9 +1271,13 @@ static uintptr_t jit_prev_page = 0, jit_prev_length = 0;
 void *scheme_malloc_gcable_code(intptr_t size)
 {
   void *p;
+
+#ifdef USE_SENORA_GC
+  p = GC_malloc_code(size);
+#else
   p = scheme_malloc(size);
   
-#if defined(MZ_CODE_ALLOC_USE_MPROTECT) || defined(MZ_JIT_USE_WINDOWS_VIRTUAL_ALLOC)
+# if defined(MZ_CODE_ALLOC_USE_MPROTECT) || defined(MZ_JIT_USE_WINDOWS_VIRTUAL_ALLOC)
   {
     /* [This chunk of code moved from our copy of GNU lightning to here.] */
     uintptr_t page, length, page_size;
@@ -1290,12 +1294,12 @@ void *scheme_malloc_gcable_code(intptr_t size)
        chunk of memory is used to compile multiple functions.  */
     if (!(page >= jit_prev_page && page + length <= jit_prev_page + jit_prev_length)) {
       
-# ifdef MZ_JIT_USE_WINDOWS_VIRTUAL_ALLOC
+#  ifdef MZ_JIT_USE_WINDOWS_VIRTUAL_ALLOC
       {
         DWORD old;
         VirtualProtect((void *)page, length, PAGE_EXECUTE_READWRITE, &old);
       }
-# else
+#  else
       {
         int r;
         r = mprotect ((void *) page, length, PROT_READ | PROT_WRITE | PROT_EXEC);
@@ -1303,7 +1307,7 @@ void *scheme_malloc_gcable_code(intptr_t size)
           scheme_log_abort("mprotect for generate-code page failed; aborting");
         }
       }
-# endif
+#  endif
 
       /* See if we can extend the previously mprotect'ed memory area towards
          higher addresses: the starting address remains the same as before.  */
@@ -1321,6 +1325,7 @@ void *scheme_malloc_gcable_code(intptr_t size)
         jit_prev_page = page, jit_prev_length = length;
     }
   }
+# endif
 #endif
 
   return p;
@@ -1683,7 +1688,7 @@ void scheme_unused_intptr(intptr_t i) { }
 extern "C" 
 {
 # endif
-  extern MZ_DLLIMPORT void GC_dump(void);
+  extern MZGC_DLLIMPORT void GC_dump(void);
 # ifdef __cplusplus
 };
 # endif
@@ -1901,6 +1906,42 @@ static void cons_onto_list(void *p)
 {
   cons_accum_result = scheme_make_pair((Scheme_Object *)p, cons_accum_result);
 }
+
+static int print_all_traced(void *p) { return 1; }
+
+static int traced_buffer_counter, traced_buffer_size;
+static void **traced_buffer;
+
+static int record_traced(void *p)
+{
+  if (traced_buffer_counter == traced_buffer_size) {
+    void **b2;
+    int new_size = (traced_buffer_size
+                    ? (2 * traced_buffer_size)
+                    : 512);
+    if (!traced_buffer) REGISTER_SO(traced_buffer);
+    b2 = scheme_malloc(sizeof(void*) * new_size);
+    memcpy(b2, traced_buffer, sizeof(void*)*traced_buffer_size);
+    traced_buffer = b2;
+    traced_buffer_size = new_size;
+  }
+
+  traced_buffer[traced_buffer_counter++] = p;
+
+  return 1;
+}
+
+static int record_traced_and_print_new(void *p)
+{
+  int i;
+
+  for (i = 0; i < traced_buffer_counter; i++) {
+    if (p == traced_buffer[i])
+      return 0;
+  }
+
+  return record_traced(p);
+}
 #endif
 
 #if MZ_PRECISE_GC
@@ -2168,6 +2209,7 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
   int trace_for_tag = 0;
   int dump_flags = 0;
   GC_for_each_found_proc for_each_found = NULL;
+  GC_print_traced_filter_proc maybe_print_traced_filter = NULL;
 # else
 #  define skip_summary 0
 #  define dump_flags 0
@@ -2185,6 +2227,10 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
 # endif
 #endif
 
+#if defined(MZ_PRECISE_GC) && MZ_PRECISE_GC_TRACE
+  maybe_print_traced_filter = print_all_traced;
+#endif
+
 #if 0
   /* Syntax-object debugging support: */
   if ((c == 1) && SCHEME_STXP(p[0])) {
@@ -2485,6 +2531,10 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
       if (tn && !strcmp(tn, s)) {
 	trace_for_tag = i;
 	dump_flags |= GC_DUMP_SHOW_TRACE;
+        if ((c > 1)
+            && SCHEME_SYMBOLP(p[1])
+            && !strcmp(SCHEME_SYM_VAL(p[1]), "new"))
+          maybe_print_traced_filter = record_traced_and_print_new;
 	break;
       }
     }
@@ -2576,6 +2626,7 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
 		      scheme_get_type_name_or_null,
 		      for_each_found,
 		      trace_for_tag, trace_for_tag,
+                      maybe_print_traced_filter,
 		      print_tagged_value,
 		      path_length_limit,
                       for_each_struct);
@@ -2665,8 +2716,9 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
     scheme_console_printf(" (dump-memory-stats 'count sym) - return number of instances of type named by sym.\n");
     scheme_console_printf("   Example: (dump-memory-stats 'count '<pair>)\n");
 # if MZ_PRECISE_GC_TRACE
-    scheme_console_printf(" (dump-memory-stats sym) - prints paths to instances of type named by sym.\n");
+    scheme_console_printf(" (dump-memory-stats sym ['new]) - prints paths to instances of type named by sym.\n");
     scheme_console_printf("   Example: (dump-memory-stats '<pair>)\n");
+    scheme_console_printf("   If 'new, all will be retrined, only new paths will be shown\n");
     scheme_console_printf(" (dump-memory-stats 'struct) - show counts for specific structure types.\n");
     scheme_console_printf(" (dump-memory-stats 'fnl) - prints not-yet-finalized objects.\n");
     scheme_console_printf(" (dump-memory-stats num) - prints paths to objects with tag num.\n");
