@@ -22,6 +22,7 @@ int (*scheme_setjmpup_relative)(Scheme_Jumpup_Buf *b, void *base,
 void (*scheme_longjmpup)(Scheme_Jumpup_Buf *b);
 void (*scheme_reset_jmpup_buf)(Scheme_Jumpup_Buf *b);
 #ifdef USE_MZ_SETJMP
+Scheme_Setjmp_Proc (*scheme_get_mz_setjmp)(void);
 int (*scheme_mz_setjmp)(mz_pre_jmp_buf b);
 void (*scheme_mz_longjmp)(mz_pre_jmp_buf b, int v);
 #endif
@@ -144,11 +145,8 @@ void (*scheme_pop_kill_action)();
 void (*scheme_set_can_break)(int on);
 void (*scheme_push_break_enable)(Scheme_Cont_Frame_Data *cframe, int on, int pre_check);
 void (*scheme_pop_break_enable)(Scheme_Cont_Frame_Data *cframe, int post_check);
-int (*scheme_with_stack_freeze)(Scheme_Frozen_Stack_Proc wha_f, void *wha_data);
-int (*scheme_frozen_run_some)(Scheme_Frozen_Stack_Proc do_f, void *do_data, int run_msecs);
-int (*scheme_is_in_frozen_stack)();
-Scheme_Object *scheme_abort_continuation_no_dws;
-Scheme_Object *scheme_call_with_composable_no_dws;
+Scheme_Object *(*scheme_abort_continuation_no_dws)(Scheme_Object *pt, Scheme_Object *v);
+Scheme_Object *(*scheme_call_with_composable_no_dws)(Scheme_Object *proc, Scheme_Object *pt);
 Scheme_On_Atomic_Timeout_Proc (*scheme_set_on_atomic_timeout)(Scheme_On_Atomic_Timeout_Proc p);
 /*========================================================================*/
 /*                              error handling                            */
@@ -952,8 +950,7 @@ int (*scheme_is_list)(Scheme_Object *obj1);
 int (*scheme_list_length)(Scheme_Object *list);
 int (*scheme_proper_list_length)(Scheme_Object *list);
 Scheme_Object *(*scheme_alloc_list)(int size);
-Scheme_Object *(*scheme_map_1)(Scheme_Object *(*f)(Scheme_Object*),
-			    Scheme_Object *l);
+Scheme_Object *(*scheme_map_1)(Scheme_Object *(*f)(Scheme_Object*), Scheme_Object *l);
 Scheme_Object *(*scheme_car)(Scheme_Object *pair);
 Scheme_Object *(*scheme_cdr)(Scheme_Object *pair);
 Scheme_Object *(*scheme_cadr)(Scheme_Object *pair);
@@ -978,6 +975,7 @@ intptr_t (*scheme_get_seconds)(void);
 intptr_t (*scheme_get_milliseconds)(void);
 double (*scheme_get_inexact_milliseconds)(void);
 intptr_t (*scheme_get_process_milliseconds)(void);
+intptr_t (*scheme_get_process_children_milliseconds)(void);
 intptr_t (*scheme_get_thread_milliseconds)(Scheme_Object *thrd);
 char *(*scheme_banner)(void);
 char *(*scheme_version)(void);
@@ -1001,7 +999,7 @@ Scheme_Hash_Table *(*scheme_get_place_table)(void);
 void *(*scheme_register_process_global)(const char *key, void *val);
 Scheme_Object *(*scheme_malloc_key)(void);
 void (*scheme_free_key)(Scheme_Object *k);
- (*scheme_jit_find_code_end)(void *p);
+void *(*scheme_jit_find_code_end)(void *p);
 void (*scheme_jit_now)(Scheme_Object *f);
 #ifndef SCHEME_EX_INLINE
 } Scheme_Extension_Table;
