@@ -41,7 +41,7 @@
 #endif
 
 #ifndef MZ_PRECISE_GC
-extern MZ_DLLIMPORT void (*GC_custom_finalize)(void);
+extern MZGC_DLLIMPORT void (*GC_custom_finalize)(void);
 #endif
 #ifndef USE_SENORA_GC
 extern int GC_is_marked(void *);
